
# $Id: dp-gibbs.R,v 1.13 2005/12/08 18:30:32 jon Exp $
# $Source: /projects/starl/cvs/dp-eb/src/dp-gibbs.R,v $

###
### Copyright (c) 2004, 2005, David M. Blei and Jon D. McAuliffe.
### This software is distributed under the terms of the GNU
### General Public License.
###

## assumes dataset         --- a list of the datapoints
##         ddata(y,thetas) --- returns density of datapoint at a theta
##         dp.eb.estimates --- returns dg0.hat, rg0.hat, alpha.hat


## first state
##

first.state <- function(b)
{
  gsamples[[b]]$theta.stars[1:N] <<- dataset;
  gsamples[[b]]$counts[1:N]      <<- 1;
  gsamples[[b]]$classes[1:N]     <<- 1:N;
}


## repeatedly updates gsamples[[b]] LAG times
##

gibbs.iter <- function(b)
{
  prob <- numeric(N+M);
  s <- gsamples[[b]];

  for (l in 1:LAG) {
    for (n in 1:N) {
      cl <- s$classes[n];

      ## subtract off counts and compute k

      k <- sum(s$counts>0);
      s$counts[cl] <- s$counts[cl] - 1;

      if (s$counts[cl] == 0) {
        s$counts[cl] <- s$counts[k];
        s$theta.stars[cl] <- s$theta.stars[k];
        s$classes[s$classes == k] <- cl;
        s$theta.stars[k] <- 0;
        s$counts[k] <- 0;
        k <- k - 1;
      }

      ## sample M reserve theta*'s

      reserve.theta.stars <- rg0.hat(M);

      ## sample class

      prob[1:k] <- s$counts[1:k] *
                     ddata(dataset[n], s$theta.stars[1:k]);
      prob[(k+1):(k+M)] <- (alpha.hat/M) *
                           ddata(dataset[n], reserve.theta.stars);

      z <- sample(k+M, 1, prob=prob[1:(k+M)]);

      if (z > k) {
        s$theta.stars[k+1] <- reserve.theta.stars[z-k];
        z <- k+1;
        s$counts[z] <- 0;
      }
      s$counts[z] <- s$counts[z] + 1;
      s$classes[n] <- z;
    }

    ## set summary statistics

    gibbs.summary[b, l, 1] <<- length(unique(s$theta.stars));
    gibbs.summary[b, l, 2] <<- max(s$counts);
  }
  gsamples[[b]] <<- s;
}


collect.B.samples <- function()
{
  gsamples[[1]] <<- gsamples[[B]];

  if (BURN.IN >= 1) {
    for (b in 1:BURN.IN)
    {
      gibbs.iter(1);
    }
  }

  t <-
    system.time(
      {
        for (b in 1:(B-1))
        {
          gibbs.iter(b);
          gsamples[[b+1]] <<- gsamples[[b]];
        }
        gibbs.iter(B);
      }
    );
  cat("gibbs time:", t, "\n");
}



plot.sample <- function(min.cex = 0.5, max.cex = 10, show.true.thetas = TRUE)
{
  if (show.true.thetas)
  {
    xlim <- axis.limits(c(dataset, thetas), p = 0.05);
  } else {
    xlim <- axis.limits(dataset, p = 0.05);
  }
  plot.new();
  plot.window(xlim=xlim, ylim=c(0,B+2));
  axis(1);
  rug(jitter(dataset));

  if (show.true.thetas)
  {
    counts <- table(thetas);
    unique <- as.numeric(names(counts));
    counts <- as.vector(counts);

    count.min <- min(sapply(gsamples,
                            function (s) { min(s$counts[s$counts>0]); }),
                     counts);

    count.max <- max(sapply(gsamples,
                            function (s) { max(s$counts[s$counts>0]); }),
                     counts);
  } else {

    count.min <- min(sapply(gsamples,
                            function (s) { min(s$counts[s$counts>0]); }));

    count.max <- max(sapply(gsamples,
                            function (s) { max(s$counts[s$counts>0]); }));
  }

  f <- function (x)
  {
    ((max.cex - min.cex) * (x - count.min) / (count.max - count.min)) + min.cex
  }

  if (show.true.thetas)
  {
    points(unique,
           y=rep(1, length(unique)), pch=20, cex=sapply(counts, f), col="red");
  }

  for (i in 1:B) {
    sample <- gsamples[[i]];
    counts <- sample$counts[sample$counts>0];
    thetas <- sample$theta.stars[sample$counts>0];
    points(x = thetas, y = rep(i+1.5, length(thetas)), pch=20,
           cex=sapply(counts, f));
  }
}
