
# $Id: eval-sim.R,v 1.16 2005/12/08 18:30:32 jon Exp $
# $Source: /projects/starl/cvs/dp-eb/src/eval-sim.R,v $

###
### Copyright (c) 2004, 2005, David M. Blei and Jon D. McAuliffe.
### This software is distributed under the terms of the GNU
### General Public License.
###

stopifnot(Sys.getenv("DPEB_LIB") != "");
source(paste(Sys.getenv("DPEB_LIB"), "/dp-model.R", sep = ""));


complete.log.likelihood <- function(s)
{
  k <- sum(s$counts>0);

  like <-
    lgamma(alpha.hat) - lgamma(N + alpha.hat) + k * log(alpha.hat) +
    sum(sapply(s$counts[s$counts>0], lgamma)) +
    sum(mapply(function (d, z)
               log(ddata(d, s$theta.stars[z])), dataset, s$classes)) +
    sum(sapply(s$theta.stars[s$counts>0],
               function (x) log(dg0.hat(x))));

  return(like);
}


multiple.kde.plot <- function(dir, show.truth.curves = TRUE)
{
  files <- list.files(dir, pattern = "round*", full.name = TRUE);
  num.graphs <- length(c(1, seq(2, length(files), by=5), length(files)));
  bitmap(file=paste(dir, "/", "kde-plots.bmp", sep = ""),
         width=10, height = 10 * num.graphs);
  par(mfrow=c(num.graphs, 1));

  ## compute limits with initial set-up (assumed to be the peakiest)

  load(files[1]);

  if (show.truth.curves)
  {
    lxg <- length(attr(eb.results$dg0.hat, "x.grid"));
    dutys <- density(unique(thetas), bw = "SJ-dpi", kernel = "gaussian",
                     n = lxg, from = attr(eb.results$dg0.hat, "x.grid")[1],
                     to = attr(eb.results$dg0.hat, "x.grid")[lxg])$y;

    ytop <-
      max(attr(eb.results$dg0.hat, "y.grid"),
          dg0(attr(eb.results$dg0.hat, "x.grid")), dutys) * 1.25;

  } else {

    ytop <-
      max(attr(eb.results$dg0.hat, "y.grid")) * 1.25;
  }

  plot.g0.hat(eb.results, ytop, title=paste("round", 0), multi = FALSE,
              show.truth.curves = show.truth.curves);

  ## make all the plots

  for (i in c(seq(2,length(files),by=5), length(files)))
  {
    load(files[i]);
    plot.g0.hat(eb.results, ytop, title=paste("round", i - 1),
                multi = TRUE, show.truth.curves = show.truth.curves);
  }
  dev.off();
}


tvd <- function(dir)
{
  files <- list.files(dir, pattern = "round*", full.name = TRUE);
  d <- length(files);

  tvd.true <- numeric(d);
  tvd.ideal <- numeric(d);
  alpha <- numeric(d);

  filename.true <- paste(dir,"/tvd.true.dat", sep="");
  filename.ideal <- paste(dir,"/tvd.ideal.dat", sep="");
  filename.alpha <- paste(dir,"/alpha.dat", sep="");

  cat("", file=filename.true);
  cat("", file=filename.ideal);
  cat("", file=filename.alpha);

  for (i in 1:d)
  {
    load(files[i]);

    tvd.true[i] <- total.variation.dist(eb.results$dg0.hat, dg0);
    cat(tvd.true[i], "\n", file = filename.true, append = TRUE);

    tvd.ideal[i] <- total.variation.dist(eb.results$dg0.hat, dg0.hat.ideal);
    cat(tvd.ideal[i], "\n", file = filename.ideal, append = TRUE);

    alpha[i] <- eb.results$alpha.hat;
    cat(alpha[i], "\n", file = filename.alpha, append = TRUE);
  }

  ## bitmap(file=paste(dir,"/",dir,"-","tvd.true.bmp", sep=""),
  ## width=11, height = 8.5);
  ## plot(tvd.true, type="b");
  ## lines(1:d, rep(total.variation.dist(dg0, dg0.hat.ideal), d))
  ## dev.off();

  ## bitmap(file=paste(dir,"/",dir,"-","tvd.ideal.bmp", sep=""),
  ## width=11, height = 8.5);
  ## plot(tvd.ideal, type="b");
  ## dev.off();

  ## bitmap(file=paste(dir,"/",dir,"-","alpha.hat.bmp", sep=""),
  ## width=11, height = 8.5);
  ## plot(alpha, type="b");
  ## dev.off();

  return(list(tvd.true = tvd.true, tvd.ideal = tvd.ideal, alpha = alpha));
}


data.likelihood <- function(dir)
{
  files <- list.files(dir, pattern = "round*", full.name = TRUE);
  likelihoods <- list();

  filename <- paste(dir,"/likelihood.dat", sep="");
  cat("", file=filename);

  for (i in 2:length(files))
  {
    load(files[i]);
    alpha.hat <<- eb.results$alpha.hat;
    dg0.hat <<- eb.results$dg0.hat;
    if ("data.sd.hat" %in% names(eb.results))
    {
      data.sd.hat <<- eb.results$data.sd.hat;
    }
    cat("computing likelihood...\n");
    likelihoods <- sapply(gsamples, complete.log.likelihood);
    cat(likelihoods, "\n", file = filename, append = TRUE);
  }
  return(likelihoods);
}


make.predictive.density <- function(s, alpha, g0.samples)
{
  counts <- as.vector(table(s$classes));
  prior <- c(s$counts[s$counts > 0], alpha);
  prior <- prior / sum(prior);
  theta.stars <- s$theta.stars[s$counts > 0];

  marginal <- function (x)
  {
    if (length(x) == 1)
    {
      mean(sapply(g0.samples, function(t) ddata(x, t)));
    } else {
      rowMeans(sapply(g0.samples, function(t) ddata(x, t)));
    }
  }

  function (x)
  {
    if (length(x) == 1)
    {
      liks <- c(sapply(theta.stars,
                       function (t) ddata(x, t)), marginal(x));
      return(sum(liks * prior));
    } else {

      liks <- cbind(sapply(theta.stars,
                           function (t) ddata(x, t)),
                    marginal(x));
      return(drop(liks %*% prior));
    }
  }
}


table.entries <- function(sim.dir, g0)
{
  # load everything

  load(paste(sim.dir,"/global-state.Rdata", sep=""), envir=.GlobalEnv);

  # compute predictive densities

  files <- list.files(sim.dir, pattern = "round*", full.name = TRUE);
  load(files[length(files)], envir=.GlobalEnv);
  rg0.hat <- eb.results$rg0.hat;
  alpha.hat <- eb.results$alpha.hat
  tvd.g0.hat <- total.variation.dist(eb.results$dg0.hat, g0$dg0);
  samples <- rg0.hat(100);
  pd.hat.lst <-
      lapply(gsamples,
             function (s)
             make.predictive.density(s, alpha.hat, samples));
  pd.hat <- function (x)
      sapply(x, function (v) mean(sapply(pd.hat.lst, function (pd) pd(v))));

  load(paste(sim.dir,"/bayes-samples.Rdata", sep=""), envir=.GlobalEnv);
  data.sd <- g0$sd/10;
  pd.conj.lst <-
      lapply(state.lst, function (s)
             make.conjugate.predictive.distribution(s, dataset, data.sd));
  mean.mean <- mean(dataset);
  mean.sd <- sd(dataset)*4;

  dg0.hat <- function (x) dnorm(x, mean = mean.mean, sd = mean.sd);
  tvd.g0.ij <- total.variation.dist(dg0.hat, g0$dg0);

  pd.ij <- function (x)
      sapply(x, function (v)  mean(sapply(pd.conj.lst, function (pd) pd(v))));

  load(paste(sim.dir,"/true-gibbs.Rdata", sep=""), envir=.GlobalEnv);
  samples <- rg0(100);
  pd.lst <-
      lapply(gsamples,
             function (s)
             make.predictive.density(s, alpha, samples));

  pd <- function (x)
      sapply(x, function (v) mean(sapply(pd.lst, function (pd) pd(v))));

  dist <- (max(dataset)  + IQR(dataset)) - (min(dataset) - IQR(dataset))
  x.grid <- seq(min(dataset) - IQR(dataset),
                max(dataset) + IQR(dataset), by=dist/500)

  cat("pd hat grid...", "\n");
  pd.hat.grid <- pd.hat(x.grid);
  cat("pd ij grid...", "\n");
  pd.ij.grid <- pd.ij(x.grid);
  cat("pd grid...", "\n");
  pd.grid <- pd(x.grid);

  tvd.pd.ij <- sum(abs(pd.ij.grid - pd.grid))/2 * (dist/500)
  tvd.pd.hat <- sum(abs(pd.hat.grid - pd.grid))/2 * (dist/500)

  cat(" tvd.pd.ij",tvd.pd.ij,"\n tvd.pd.hat",
      tvd.pd.hat,"\n tvd.g0.ij",tvd.g0.ij,"\n tvd.g0.hat",tvd.g0.hat,
      file=paste(sim.dir,"/table.results.dat",sep=""))

  return(list(pd.hat = pd.hat, pd.ij = pd.ij, pd = pd,
              pd.grid = pd.grid, pd.ij.grid = pd.ij.grid,
              pd.hat.grid = pd.hat.grid, dist = dist,
              x.grid = x.grid, tvd.g0.hat = tvd.g0.hat, dist=dist,
              tvd.g0.ij = tvd.g0.ij))
}
