
# $Id: plots.R,v 1.8 2005/12/08 18:30:32 jon Exp $
# $Source: /projects/starl/cvs/dp-eb/src/plots.R,v $

###
### Copyright (c) 2004, 2005, David M. Blei and Jon D. McAuliffe.
### This software is distributed under the terms of the GNU
### General Public License.
###

multi.kde.plot <- function(dirs, names)
{
  par(mfcol=c(4,length(dirs)), oma = c(0, 5, 5, 0), mar = c(3.1, 2.1, 0, 2.1))
  mapply(
         function (dir, name) {
           load(paste(dir, "/global-state.Rdata", sep=""), envir=.GlobalEnv)
           load(paste(dir, "/round=000.Rdata", sep=""))
           ytop <- max(attr(eb.results$dg0.hat, "y.grid")) * 1.25;
           grayscale.plot.g0.hat(eb.results, ytop, multi = FALSE, title = "")
           if (dir == dirs[[1]])
               mtext("round 0", side = 2, line = 4);
           mtext(name, side = 3, line = 2);
           load(paste(dir, "/round=005.Rdata", sep=""))
           grayscale.plot.g0.hat(eb.results, ytop, title = "")
           if (dir == dirs[[1]])
             mtext("round 5", side = 2, line = 4);
           load(paste(dir, "/round=199.Rdata", sep=""))
           grayscale.plot.g0.hat(eb.results, ytop, title = "")
           if (dir == dirs[[1]])
             mtext("round 199", side = 2, line = 4);

           ## add the tvd between the ideal and true as a dotted solid line

           tvd.true <- scan(paste(dir, "/tvd.true.dat", sep=""));
           tvd.ideal <- scan(paste(dir, "/tvd.ideal.dat", sep=""));
           tvd.true.ideal <- total.variation.dist(dg0, dg0.hat.ideal);
           tvd.true.ideal <- rep(tvd.true.ideal, length(tvd.true));
           multi.plot(list(tvd.true, tvd.ideal, tvd.true.ideal),
                      lty=list("dashed", "solid", "solid"), lwd=1.5,
                      col=list("black", "black", "gray"));
           box();
           if (dir == dirs[[1]])
               mtext("tvd", side = 2, line = 4);
         }, dirs, names);
}

## pdf(file="~/examples.pdf", height=6, width=6)
## multi.kde.plot(list("/home/blei/projects/dp-eb/src/sims/rep=01.g0=1.alpha=5.sdscale=10/", "/home/blei/projects/dp-eb/src/sims/rep=01.g0=2.alpha=5.sdscale=10/", "/home/blei/projects/dp-eb/src/sims/rep=01.g0=4.alpha=5.sdscale=10/"), list("N(0,1)", "t(3)", "mixture of two t(5)'s"))



multi.flow.plot <- function(dir, rounds)
{
  par(mfcol=c(1,length(rounds)),
      oma = c(0, 0, 0, 0), mar = c(3.1, 2.1, 0, 2.1))

  load(paste(dir, "/global-state.Rdata", sep=""), envir=.GlobalEnv)

  lapply(rounds, function (round) {
    load(paste(dir, round, sep=""), envir=.GlobalEnv);
    plot.sample();
  });
}

## pdf(file="~/gibbs-plot.pdf");
## dir <- "/home/blei/projects/dp-eb/src/sims/rep=01.g0=1.alpha=5.sdscale=10/"
## multi.flow.plot(dir, list("round=001.Rdata", "round=020.Rdata", "round=120.Rdata"))


make.pd.objs <- function(dir, round)
{
  load(paste(dir, "/global-state.Rdata.gz", sep = ""), envir = .GlobalEnv);
  load(paste(dir, "/round=", round, ".Rdata.gz", sep = ""),
       envir = .GlobalEnv);

  data.sd.hat <<- eb.results$data.sd.hat;

  pd.g0.samples <- eb.results$rg0.hat(100);
  pd.lst <- lapply(gsamples,
                   function (gs)
                     make.predictive.density(gs, eb.results$alpha.hat,
                                             pd.g0.samples));
  pd <- function(x)
  {
    if (length(x) == 1)
    {
      return(mean(sapply(pd.lst, function(pdl) pdl(x))));
    } else {
      return(drop(rowMeans(sapply(pd.lst, function(pdl) pdl(x)))));
    }
  }

  rds   <- range(dataset);
  iqrds <- IQR(dataset);
  x.grid <- seq(rds[1] - iqrds, rds[2] + iqrds, length = 500);
  y.grid <- pd(x.grid);
  attr(pd, "x.grid") <- x.grid;
  attr(pd, "y.grid") <- y.grid;

  pd.lines <- sapply(pd.lst, function(pdl) pdl(x.grid));

  save(pd.g0.samples, pd.lst, pd, pd.lines,
       file = paste(dir, "/round=", round, ".pd-objs.Rdata", sep = ""));
}


# data.analysis.result.plot(dirs = c("runs/galaxies.alpha-init=std",                                                  "runs/stamp.alpha-init=std"),                                           rounds = rep("200", 2),                                                          titles = c("Galaxy", "Stamp"),                                                   xlabs = c("km/s (thousands)", "mm (hundredths)"),                                rescalers = c(1000, .01),                                                        out.file =                                                                       "plots.stat-and-comp/data-analysis-results")

data.analysis.result.plot <- function(dirs, rounds, titles, xlabs, rescalers,
                                      out.file)
{
  pdf(file = paste(out.file, ".pdf", sep = ""), width = 5, height = 5);
#  x11(width = 5, height = 5);
  par(mfrow = c(2,2), oma = c(0, 0, 0, 0), mar = c(5.1, 2.1, 2, 1.1));
  mapply(function(dir, round, title, xlab, rescaler)
         {
           load(paste(dir, "/global-state.Rdata.gz", sep = ""),
                envir = .GlobalEnv);
           load(paste(dir, "/round=", round, ".Rdata.gz", sep = ""),
                envir = .GlobalEnv);
           load(paste(dir, "/round=", round, ".pd-objs.Rdata.gz", sep = ""),
                envir = .GlobalEnv);

           multi <- ifelse(round == "000", FALSE, TRUE);

           xlim <- range(attr(eb.results$dg0.hat, "x.grid")) / rescaler;
           xlim[1] <- min(xlim[1], min(attr(pd, "x.grid")) / rescaler);
           xlim[2] <- max(xlim[2], max(attr(pd, "x.grid")) / rescaler);

           ytop <- max(pd.lines,
                       attr(eb.results$dg0.hat, "y.grid")) * rescaler;
           if (multi)
           {
             ytop <-
               max(ytop,
                   sapply(eb.results$kde.results,
                          function(kder) max(kder$d$y) * rescaler));
           }

           plot.g0.hat(eb.results, xlim = xlim, ytop = ytop,
                       rescaler = rescaler,
                       title = substitute(t~~hat(G)[0], list(t=title)),
                       multi = multi,
                       xlab = xlab,
                       show.truth.curves = FALSE);

           plot(0, 0, type = "n", xlim = xlim, ylim = c(0, ytop),
                main = paste(title, "PD"), font.main = 1, xlab = "");
           mtext(xlab, side = 1, line = 2.3);
#           plot.new();
#           plot.window(xlim = xlim, ylim = c(0, ytop));
#           axis(1); axis(2);
#           title(main = paste(title, "PD"), font.main = 1);

           envelope <- apply(pd.lines, 1, range) * rescaler;
           segments(attr(pd, "x.grid") / rescaler, envelope[1,],
                    attr(pd, "x.grid") / rescaler, envelope[2,],
                    col = "lightgray");

           lines(attr(pd, "x.grid") / rescaler,
                 attr(pd, "y.grid") * rescaler, col = "black", lwd = 2);
         }, dirs, rounds, titles, xlabs, rescalers);
  dev.off();
}
