
# $Id: utils.R,v 1.7 2005/12/08 18:30:32 jon Exp $
# $Source: /projects/starl/cvs/dp-eb/src/utils.R,v $

###
### Copyright (c) 2004, 2005, David M. Blei and Jon D. McAuliffe.
### This software is distributed under the terms of the GNU
### General Public License.
###


total.variation.dist <- function(f1, f2, lower = -Inf, upper = Inf)
{
  ires <- integrate(function(x) abs(f1(x) - f2(x)),
                    lower = lower, upper = upper);

  if (ires$message != "OK")
  {
    cat("total.variation.dist: WARNING: integrate failed: ", ires$message,
        ", absolute error = ", ires$abs.error, "\n", sep = "");
    return(NaN);
  }

  return(ires$value / 2);
}


axis.limits <- function(x.vct, p=0.05)
{
  xmin <- min(x.vct);
  xmax <- max(x.vct);
  xdiff <- xmax-xmin;
  xmin <- xmin-xdiff*p;
  xmax <- xmax+xdiff*p;
  return(c(xmin, xmax));
}


multi.plot <- function(y.lst, x.lst=lapply(y.lst, function (x) 1:length(x)),
                       lty, col, lwd=1, xlab="", ylab="", main="")
{
  ylim <- axis.limits(unlist(y.lst));
  xlim <- axis.limits(unlist(x.lst));

  plot.new();
  plot.window(xlim = xlim, ylim = ylim);
  title(xlab = xlab, ylab = ylab, main = main)
  axis(1); axis(2);

  for (i in 1:length(y.lst))
  {
    lines(y.lst[[i]], type="l", lty=lty[[i]], col=col[[i]], lwd=lwd);
  }
}
